"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setBehavior = exports.objectID = exports.helper = exports.getBehaviorArgs = exports.setDisplay = exports.behaviorCustom = exports.noRegisterSetValAction = exports.setValAction = exports.setValArgs = exports.setVal = exports.getVal = exports.setInline = exports.setMut = exports.extendBehavior = exports.createBehavior = void 0;
const util_1 = require("../util");
const Behavior_1 = require("../types/Behavior");
/**
 * Creates a new behavior.
 * Usage: createBehavior!(name: string, body: Body);
 */
exports.createBehavior = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const body = (0, util_1.getBlock)(args, state, 1, "A behavior definition is required!");
        if (state.graphgame.behaviors.hasOwnProperty(name))
            throw new Error("A behavior with the name \"" + name + "\" already exists!");
        state.graphgame.behaviors[name] = new Behavior_1.Behavior(name);
        state.graphgame.currentBehavior = name;
        return body;
    }
};
/**
 * Add to an already existing behavior.
 * Usage: extendBehavior!(name: string, body: Body);
 */
exports.extendBehavior = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const body = (0, util_1.getBlock)(args, state, 1, "A behavior definition is required!");
        if (!state.graphgame.behaviors.hasOwnProperty(name))
            throw new Error("A behavior with the name \"" + name + "\" does not exist!");
        state.graphgame.currentBehavior = name;
        return body;
    }
};
/**
 * Marks a behavior's variable as mutable (changable after compilation).
 * Usage: setMut!(variableName: string);
 */
exports.setMut = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const varName = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase();
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].add((id) => prefix + `selectID!(${id});
        setMutSelect!("${(0, util_1.getFullVariableName)(varName, name)}");
        selectID!();` + suffix);
        return "";
    }
};
/**
 * Marks a behavior's variable as mutable (changable after compilation), but inlines it instead of exporting it.
 * Usage: setInline!(variableName: string);
 */
exports.setInline = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const varName = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase();
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].add((id) => prefix + `selectID!(${id});
        setInlineSelect!("${(0, util_1.getFullVariableName)(varName, name)}");
        selectID!();` + suffix);
        return "";
    }
};
/**
 * Get the value of a behavior's variable. A boolean can be supplied to get the currently saved value instead of the current value.
 * Usage: getVal!(variableName: string, saved?: boolean);
 */
exports.getVal = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.expressionCheck)(context);
        const name = state.graphgame.currentBehavior;
        const varName = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase();
        const saved = (0, util_1.getBoolean)(args, state, 1);
        const fullName = (0, util_1.getFullVariableName)(varName, name);
        (0, util_1.behaviorCheck)(state, name);
        (0, util_1.objectVarCheck)(state, state.graphgame.lastObjectBehaviorId, fullName);
        const value = (0, util_1.getSemiMut)(state, state.graphgame.lastObjectBehaviorId, fullName);
        if (saved && value.isMut())
            return value.name(true);
        return value.get();
    }
};
/**
 * Set the value of a behavior's variable (during compilation). This must be used before a variable is marked as mutable.
 * Usage: setVal!(variableName: string, val: number);
 */
exports.setVal = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const varName = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase();
        const val = (0, util_1.getNum)(args, state, 1, "A value is required!");
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].add((id) => prefix + `selectID!(${id});
        setValSelect!("${(0, util_1.getFullVariableName)(varName, name)}", ${val});
        selectID!();` + suffix);
        return "";
    }
};
/**
 * Set the value of a behavior's variable (during compilation) using a zero-based argument that was passed to the behavior when it was added to the GameObject. This must be used before a variable is marked as mutable.
 * Usage: setValArgs!(variableName: string, arg: number, defaultVal?: number);
 */
exports.setValArgs = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const varName = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase();
        const idx = (0, util_1.getNum)(args, state, 1, "An argument index is required!");
        const defaultVal = (0, util_1.getNum)(args, state, 2);
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].add((id) => {
            let val;
            if (state.graphgame.lastObjectBehaviorArgs.length <= idx) {
                if (defaultVal == null)
                    throw new Error("The behavior \"" + name + "\" requires at least " + (idx + 1) + " arguments!");
                else
                    val = defaultVal;
            }
            else {
                val = state.graphgame.lastObjectBehaviorArgs[idx];
            }
            if (typeof (val) === "string")
                val = `"${val}"`;
            else if (typeof (val) === "object" && val["block"])
                val = val["value"];
            else
                val = val.toString();
            return prefix + `selectID!(${id});
            setValSelect!("${(0, util_1.getFullVariableName)(varName, name)}", ${val});
            selectID!();` + suffix;
        });
        return "";
    }
};
/**
 * Set the value of a behavior's variable on update (during runtime). This must be used after a variable is marked as mutable.
 * Usage: setValAction!(variableName: string, body: Body, priority?: number = 0, exported?: boolean = false, variable?: boolean = false);
 */
exports.setValAction = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const varName = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase();
        const body = (0, util_1.getBlock)(args, state, 1, "An action body is required!");
        const priority = (0, util_1.getNum)(args, state, 2) || 0;
        const exported = (0, util_1.getBoolean)(args, state, 3);
        const variable = (0, util_1.getBoolean)(args, state, 4);
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].addPost((id) => prefix + `selectID!(${id});
        setValActionSelect!("${(0, util_1.getFullVariableName)(varName, name)}", {const ${(0, util_1.getShortVariableName)(varName)} = ${(0, util_1.getFullVariableName)(varName, name)};setBehavior!("${name}");${body}setBehavior!();}, ${exported ? "true" : "false"}, ${variable ? "true" : "false"});
        selectID!();` + suffix, priority);
        return "";
    }
};
/**
 * Create an action that sets the value of a behavior's variable (during runtime). This must be run manually (or as the click property of an object). This must be used after a variable is marked as mutable.
 * Usage: noRegisterSetValAction!(variableName: string, body: Body, actionName?: string, priority?: number = 0);
 */
exports.noRegisterSetValAction = {
    function: (args, state, context) => {
        var _a;
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const varName = (0, util_1.getString)(args, state, 0, "A variable name is required!").trim().toLowerCase();
        const body = (0, util_1.getBlock)(args, state, 1, "An action body is required!");
        const actionName = (_a = (0, util_1.getString)(args, state, 2)) === null || _a === void 0 ? void 0 : _a.trim().toLowerCase();
        const priority = (0, util_1.getNum)(args, state, 3) || 0;
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].addPost((id) => prefix + `selectID!(${id});
        noRegisterSetValActionSelect!("${(0, util_1.getFullVariableName)(varName, name)}", {const ${(0, util_1.getShortVariableName)(varName)} = ${(0, util_1.getFullVariableName)(varName, name)};setBehavior!("${name}");${body}setBehavior!();}${actionName ? ", \"" + actionName + "\"" : ""});
        selectID!();` + suffix, priority);
        return "";
    }
};
/**
 * Create custom declarations on the behavior. This is ideal for graphs, polygons, points, etc, but should not be used for named declarations ((functions, consts, etc).
 * Usage: behaviorCustom!(body: Body, priority?: number = 0);
 */
exports.behaviorCustom = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const body = (0, util_1.getBlock)(args, state, 0, "A body is required!");
        const priority = (0, util_1.getNum)(args, state, 1) || 0;
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].addPost(id => {
            return prefix + `selectID!(${id});
            setBehavior!("${name}");
            ${body}
            setBehavior!();
            selectID!();` + suffix;
        }, priority);
        return "";
    }
};
/**
 * Allows modifying the display properties of any object that this is attached to.
 * Usage: setDisplay!(body: Body, priority?: number = 0);
 */
exports.setDisplay = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const body = (0, util_1.getBlock)(args, state, 0, "An action body is required!");
        const priority = (0, util_1.getNum)(args, state, 1) || 0;
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].addDisplay(prefix + body + suffix, priority);
        return "";
    }
};
/**
 * Gets a zero-based argument that was passed to the behavior when it was added to the GameObject.
 * Usage: getBehaviorArgs!(arg: number);
 */
exports.getBehaviorArgs = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        const idx = (0, util_1.getNum)(args, state, 0, "An argument index is required!");
        if (state.graphgame.lastObjectBehaviorArgs.length <= idx)
            throw new Error("This behavior requires at least " + (idx + 1) + " arguments!");
        let val = state.graphgame.lastObjectBehaviorArgs[idx];
        if (typeof (val) === "string")
            val = `"${val}"`;
        else if (typeof (val) === "object" && val["block"])
            val = val["value"];
        else
            val = val.toString();
        return val;
    }
};
/**
 * Allows creating helper methods at certain priorities, and which are only output if the behavior they are attached to is used.
 * Usage: helper!(body: Body, priority?: number = 0);
 */
exports.helper = {
    function: (args, state, context) => {
        var _a;
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = state.graphgame.currentBehavior;
        const body = (0, util_1.getBlock)(args, state, 0, "An action body is required!");
        const priority = (_a = (0, util_1.getNum)(args, state, 1)) !== null && _a !== void 0 ? _a : -10000;
        (0, util_1.behaviorCheck)(state, name);
        const prefix = "setFile!(\"" + state.logimat.files[state.logimat.files.length - 1] + "\");";
        const suffix = "setFile!();";
        state.graphgame.behaviors[name].addHelper(prefix + body + suffix, priority);
        return "";
    }
};
/**
 * Gets the ID of the object that this behavior is currently attached to.
 * Usage: objectID!();
 */
exports.objectID = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.expressionCheck)(context);
        return state.graphgame.lastObjectBehaviorId.toString();
    }
};
/**
 * For internal use only.
 */
exports.setBehavior = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerInnerCheck)(context);
        state.graphgame.currentBehavior = (0, util_1.getString)(args, state, 0) || "";
        return "";
    }
};
//# sourceMappingURL=component.js.map