"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemiMutable = void 0;
class SemiMutable {
    /**
     * Creates a SemiMutable, which stores an immutable value until it is made mutable.
     * Once it is made mutable, it gets converted to a variable, and all mutations can
     * only be done from Desmos.
     * @param varName {string} - is the variable name of this variable.
     * @param id {number} - is the ID of the GameObject that this is attached to.
     * @param val {number} - is the default value.
     * @param state {TemplateState} - is the current state.
     */
    constructor(varName, id, val, state) {
        this.mutable = false;
        this.inlined = false;
        this.incrementID = 0;
        this.variableIDs = [];
        this.varName = varName;
        this.id = id;
        this.val = val;
        this.exportID = state.graphgame.nextVariableId++;
    }
    /**
     * Makes a SemiMutable mutable.
     */
    mut() {
        if (this.inlined && !this.mutable)
            throw new Error("An inlined variable cannot be marked as mutable!");
        this.mutable = true;
    }
    /**
     * Returns if this is mutable.
     */
    isMut() {
        return this.mutable;
    }
    /**
     * Makes a SemiMutable inlined.
     */
    inline() {
        if (!this.inlined && this.mutable)
            throw new Error("A mutable variable cannot be marked as inlined!");
        this.mutable = true;
        this.inlined = true;
    }
    /**
     * Returns if this is inlined.
     */
    isInline() {
        return this.inlined;
    }
    /**
     * Sets the value.
     */
    set(val) {
        if (this.mutable)
            throw new Error("A mutable variable can only be set from an action. Consider setting its value before making it mutable.");
        this.val = val;
    }
    /**
     * Gets the value. If supplied, an offset is added to the increment.
     */
    get(offset = 0) {
        if (this.mutable)
            return this.name(false, offset) + (this.incrementID + offset > 0 && !this.variableIDs.includes(this.incrementID + offset) ? "()" : "");
        return this.val.toString();
    }
    /**
     * Gets the variable name. If supplied, an offset is added to the increment.
     */
    name(noIncrement = false, offset = 0) {
        if (!this.mutable)
            throw new Error("mut() must be called before the name can be accessed.");
        const incrementVal = Math.max(0, this.incrementID + offset);
        return "g_v" + this.exportID + (noIncrement ? "" : (incrementVal ? "n" + incrementVal : ""));
    }
    /**
     * Increment the value's identifier.
     * @param variable {boolean} - is a value indicating whether this increment should be treated as a variable and not a function.
     */
    increment(variable = false) {
        if (!this.mutable)
            throw new Error("A variable must be marked as mutable before it can be used in an action!");
        this.incrementID++;
        if (variable)
            this.variableIDs.push(this.incrementID);
    }
    /**
     * Decrement the value's identifier.
     */
    decrement() {
        if (!this.mutable)
            throw new Error("A variable must be marked as mutable before it can be used in an action!");
        this.incrementID--;
    }
}
exports.SemiMutable = SemiMutable;
//# sourceMappingURL=SemiMutable.js.map